use std::{collections::HashMap, io::{BufRead, BufReader, Read}};

use regex_lite::Regex;

use crate::F32Vertex;


pub fn parse_file(file: &mut dyn Read) -> (Vec<[u32;3]>,Vec<F32Vertex>) {
    let mut reader = BufReader::new(file);
    let mut line = String::new();

    let v_regex = Regex::new(r"^v (-?[\d\.]+)+\s+(-?[\d\.]+)+\s+(-?[\d\.]+)+\s+(-?[\d\.]+)+\s+(-?[\d\.]+)+\s+(-?[\d\.]+)+").unwrap();
    let vn_regex = Regex::new(r"^vn (-?[\d\.]+)+\s+(-?[\d\.]+)+\s+(-?[\d\.]+)+").unwrap();
    let vp_regex = Regex::new(r"^vp (-?[\d\.]+)+\s+(-?[\d\.]+)+").unwrap();
    let f_regex = Regex::new(r"^f (\d+)\/(\d*)\/(\d*)\s+(\d+)\/(\d*)\/(\d*)\s+(\d+)\/(\d*)\/(\d*)").unwrap();

    let mut v_lines = Vec::<(f32,f32,f32,f32,f32,f32)>::new();
    let mut vn_lines = Vec::<(f32,f32,f32)>::new();
    let mut vp_lines = Vec::<(f32,f32)>::new();
    let mut faces = Vec::<[(u32,u32,u32);3]>::new();

    loop {
        line.clear();
        let len = reader.read_line(&mut line).unwrap();
        if len == 0 { break; }

        if let Some(caps) = v_regex.captures(&line) {
            v_lines.push((
                caps.get(1).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(2).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(3).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(4).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(5).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(6).unwrap().as_str().parse().unwrap_or(0.0),
            ));
        } else if let Some(caps) = vn_regex.captures(&line) {
            vn_lines.push((
                caps.get(1).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(2).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(3).unwrap().as_str().parse().unwrap_or(0.0),
            ));
        } else if let Some(caps) = vp_regex.captures(&line) {
            vp_lines.push((
                caps.get(1).unwrap().as_str().parse().unwrap_or(0.0),
                caps.get(2).unwrap().as_str().parse().unwrap_or(0.0),
            ));
        } else if let Some(caps) = f_regex.captures(&line) {
            faces.push([
                (
                    caps.get(1).unwrap().as_str().parse().unwrap_or(0),
                    caps.get(2).unwrap().as_str().parse().unwrap_or(0),
                    caps.get(3).unwrap().as_str().parse().unwrap_or(0),
                ),
                (
                    caps.get(4).unwrap().as_str().parse().unwrap_or(0),
                    caps.get(5).unwrap().as_str().parse().unwrap_or(0),
                    caps.get(6).unwrap().as_str().parse().unwrap_or(0),
                ),
                (
                    caps.get(7).unwrap().as_str().parse().unwrap_or(0),
                    caps.get(8).unwrap().as_str().parse().unwrap_or(0),
                    caps.get(9).unwrap().as_str().parse().unwrap_or(0),
                )
            ]);
        } else {
            println!("{}",line);
        }
    }

    let mut vt_map = HashMap::<(u32,u32,u32),u32>::new();
    let mut new_vt = Vec::<F32Vertex>::new();
    let mut new_face = Vec::<[u32;3]>::new();

    for face in faces {
        new_face.push(face.map(|vt| {
            match vt_map.get(&vt) {
                Some(i) => *i,
                None => {
                    // create a new vertex
                    let i = u32::try_from(new_vt.len()).unwrap();
                    let pos = if vt.0 != 0 {v_lines[(vt.0-1) as usize]} else {(0.,0.,0.,0.,0.,0.)};
                    let tex = if vt.1 != 0 {vp_lines[(vt.1-1) as usize]} else {(0.,0.)};
                    let nrm = if vt.2 != 0 {vn_lines[(vt.2-1) as usize]} else {(0.,0.,0.)};
                    new_vt.push(F32Vertex {
                        x: pos.0, y: pos.1, z: pos.2, u: tex.0, v: tex.1, nx: nrm.0, ny: nrm.1, nz: nrm.2, shade: pos.4 // shade is green channel
                    });
                    vt_map.insert(vt,i);
                    i
                }
            }
        }));
    }

    return (new_face,new_vt);
}